# coding: utf-8
#!/usr/bin/env python2

import sys

import os
import rospy
import rosbag
import csv
import cv2
from cv_bridge import CvBridge, CvBridgeError
import argparse
import struct
import subprocess
from subprocess import Popen, PIPE
import numpy as np

bridge = CvBridge()

bag_start_time=0
bag_end_time=0

def parsebag(f, output_dir, target_topic):
    global bag_start_time
    global bag_end_time
    rosbag_name = f.split("/")[-1].split(".")[0]
    output_path = os.path.join(output_dir, rosbag_name) 
    camera_exist_flag=False

    with rosbag.Bag(f, 'r') as bag:
        bag_start_time = bag.get_start_time()
        bag_end_time = bag.get_end_time()
        count = 0
        start_time = bag.get_start_time()
        end_time = bag.get_end_time()
        duration = end_time - start_time
        print('duration',duration)
        topics = bag.get_type_and_topic_info()[1].keys()
        if target_topic in topics:        
            for topic, msg, t in bag.read_messages(topics=[target_topic]):
                camera_exist_flag=True
                img = bridge.imgmsg_to_cv2(msg, 'bgr8')
                img_resized = cv2.resize(img, (1920, 1080))
                # img = bridge.compressed_imgmsg_to_cv2(msg, 'bgr8')
                timestr = msg.header.stamp.to_nsec()
                image_name = str(timestr)[:10]+'.'+str(timestr)[10:] + ".jpg"
                if not os.path.exists(output_path + '_' + 'image'):
                    os.makedirs(output_path + '_' + 'image')
                output_path_img = os.path.join(output_path + '_' + 'image', image_name)
                cv2.imwrite(output_path_img, img_resized)
                # print(count)
                count = count + 1   
            
    return count , duration, camera_exist_flag

def parse(input_dir,output_dir):

    # input_dir=sys.argv[1]
    # output_dir = sys.argv[2]
    
    #input_dir='/media/dell/HIKSEMI1/2024-12-04-10-32-38.bag'
    #output_dir ='/media/dell/HIKSEMI1'
    bagname=input_dir.split('/')[-1].split('.bag')[0]
    output_dir=os.path.join(output_dir, bagname)
    topic = '/image_raw'  # /camera/color/image_raw

    num_count, duration ,flag = parsebag(input_dir, output_dir, topic)
    
    
    if flag:
        dirt = os.path.join(output_dir, input_dir.split('/')[-1].split('.')[0] + '_' + 'image')
        '''
        下面这段代码是用ffmpeg通过动态计算视频帧率将图片拼接成视频的代码
        
        bagtime = duration
        hz = str(float(num_count) / bagtime)
        print('bagtime=',bagtime)
        print('num_count=',num_count)
        
        command = ['ffmpeg', '-f', 'image2', '-r', hz, '-pattern_type', 'glob', '-i'] + [dirt + '/*.jpg'] + ['-tag:v',
                                                                                                            'avc1',
                                                                                                            '-y'] + [
                    dirt + '/camera.mp4']
        p = Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        p.communicate()
        print('done')
        '''
        
        '''
        下面这段代码是利用图像插值算法，通过指定一个频率将图像拼接成视频的代码
        '''
        #指定视频帧率
        hz=10.0
        
        # 定义图片文件夹路径
        folder_path = dirt

        #根据rosbag的起止时间生成黑色图片
        if bag_end_time!=0 and bag_end_time!=0:
            black_image = np.zeros((512, 512, 3), dtype=np.uint8)
            cv2.imwrite(os.path.join(dirt,str(bag_start_time)+'.jpg'), black_image)
            cv2.imwrite(os.path.join(dirt,str(bag_end_time)+'.jpg'), black_image)
        
        # 获取文件夹中的所有图片文件
        image_files = [f for f in os.listdir(folder_path) if f.endswith('.jpg')]

        # 对文件名进行排序，以便按顺序处理图片
        image_files.sort()

        # 读取第一张图片
        first_image = cv2.imread(os.path.join(folder_path, image_files[0]))

        # 获取图片的形状
        image_shape = first_image.shape

        # 创建视频写入对象
        video_path=os.path.join(dirt,'camera.mp4')
        video_writer = cv2.VideoWriter(video_path, cv2.VideoWriter_fourcc('m', 'p', '4', 'v'), hz, (image_shape[1], image_shape[0]))

        # 遍历图片文件列表
        for i in range(len(image_files) - 1):
            # 读取当前图片和下一张图片
            current_image = cv2.imread(os.path.join(folder_path, image_files[i]))
            next_image = cv2.imread(os.path.join(folder_path, image_files[i + 1]))

            # 获取当前图片和下一张图片的时间戳
            current_timestamp = float(image_files[i].split('.jpg')[0]) 
            next_timestamp = float(image_files[i + 1].split('.jpg')[0]) 

            # 计算时间间隔
            time_interval = next_timestamp - current_timestamp
            #print(time_interval)

            # 根据实际时间间隔调整循环次数
            video_writer.write(current_image)
            if int(time_interval * hz)>1.5:
                for _ in range(int(time_interval * hz)):     
                    video_writer.write(current_image)

        # 释放资源
        video_writer.release()
        if os.path.exists(video_path):
            video_output_path=os.path.join(folder_path,'camera_new.mp4')
            command = ["ffmpeg", "-i", video_path, "-c:v", "libx264", video_output_path]
            p = Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            p.communicate()
            os.remove(video_path)
            if os.path.exists(video_output_path):
                os.rename(video_output_path, video_path)
                print('done')
    return flag,dirt
