#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Wed Nov  1 13:38:28 2023

@author: dell
"""
from __future__ import print_function
import subprocess
from subprocess import Popen, PIPE
import os
import sys
import math
import cv2
import numpy as np
import shutil
import rosbag
from multiprocessing import Pool
import datetime


def draw_points(pic, filename):
    colormap = np.array([[128, 130, 120],
                         [235, 0, 205],
                         [0, 215, 0],
                         [235, 155, 0]]) / 255.0
    f = open(filename, "r")
    ff = f.readlines()[11:]

    for i in range(0, len(ff)):
        point = ff[i].split()[:4]
        point_x = float(point[0])
        point_y = -float(point[1])
        point_z = float(point[2])
        points_intensity = float(point[2])
        if math.isnan(point_x):
            pass
        else:
            cv2.circle(pic, (int(800 - point_x * 10), int(400 - point_y * 10)), 0,
                       colormap[int(points_intensity) % colormap.shape[0]] * 255, 2)


def process_file(file):
    root = file[0]
    inputFilePath = root + "/" + file[1]
    tempFilePath = root[:-3] + "pcd_ascii/" + file[1]
    outputFilePath = root[:-3] + "jpg/" + file[1][:-3] + "jpg"

    command = "pcl_convert_pcd_ascii_binary " + inputFilePath + " " + tempFilePath + " 0"
    os.system(command)

    if not os.path.exists(tempFilePath):
        return

    pic = np.zeros([800, 1600, 3])
    draw_points(pic, tempFilePath)
    cv2.imwrite(outputFilePath, pic)


def parse(input_dir, output_dir):
    s_time = datetime.datetime.now()
    # input_dir = sys.argv[1]
    # input_dir='/media/dell/HIKSEMI/2024-09-06-11-54-36.bag'
    # output_dir = sys.argv[2]
    # output_dir = '/media/dell/HIKSEMI'

    bag_name = input_dir.split('/')[-1].split('.')[0]
    output_dir = os.path.join(output_dir, bag_name + '_pcd' + '/pcd')
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    with rosbag.Bag(input_dir, 'r') as bag:
        bag_start_time = bag.get_start_time()
        bag_end_time = bag.get_end_time()
        print('bag_start_time=', bag_start_time)
        print('bag_end_time=', bag_end_time)
        num_count = 0
        for topic, msg, t in bag.read_messages(topics=['/velodyne_points']):
            # /camera/depth/points   # /scan_map_icp_amcl_node/scan_point_transformed
            num_count += 1
        start_time = bag.get_start_time()
        end_time = bag.get_end_time()
        # 计算rosbag的持续时间
        duration = end_time - start_time

    bagtime = duration
    hz = str(float(float(num_count) / bagtime))
    print(hz)

    #######################解析rosbag中的点云point_concat话题，获得原始pcd文件#######################
    command = ['rosrun', 'pcl_ros', 'bag_to_pcd'] + [input_dir] + ['/velodyne_points'] + [output_dir]
    command = ' '.join(command)
    os.system(command)
    file1 = os.path.join(output_dir[0:-4], 'pcd_ascii')
    file2 = os.path.join(output_dir[0:-4], 'jpg')
    file3 = os.path.join(output_dir[0:-4], 'pcd')
    if not os.path.exists(file1):
        os.makedirs(file1)
    if not os.path.exists(file2):
        os.makedirs(file2)

    #######################将原始pcd文件转化成pcd-ascii格式,并生成jpg文件#######################
    # Create a list of files to be processed
    files_to_process = []
    for root, dirs, files in os.walk(output_dir[:-4]):
        for file in files:
            if file.endswith(".pcd"):
                files_to_process.append((root, file))

    num_processes = 4

    pool = Pool(num_processes)
    pool.map(process_file, files_to_process)
    pool.close()
    pool.join()

    #######################将转化的点云jpg合成视频#######################
    jpg_list = os.listdir(file2)
    if not jpg_list == []:
        '''
        下面这段代码是用ffmpeg通过动态计算视频帧率将图片拼接成视频的代码
        command = ['ffmpeg', '-f', 'image2', '-r', hz, '-pattern_type', 'glob', '-i'] + ['"jpg/*.jpg"'] + ['-tag:v',
                                                                                                           'avc1',
                                                                                                           '-y'] + [
                      'pcd_overlook.mp4']
        result_string = " ".join(command)
        p = Popen(result_string, shell=True, cwd=output_dir[0:-4] + '/', stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        p.wait()


        '''

        '''
        下面这段代码是利用图像插值算法，通过指定一个频率将图像拼接成视频的代码
        '''
        # 指定视频帧率
        hz = 10.0

        # 定义图片文件夹路径
        folder_path = file2
        '''
        #根据rosbag的起止时间生成黑色图片
        black_image = np.zeros((512, 512, 3), dtype=np.uint8)
        cv2.imwrite(os.path.join(file2,str(bag_start_time)+'.jpg'), black_image)
        cv2.imwrite(os.path.join(file2,str(bag_end_time)+'.jpg'), black_image)
        '''
        # 获取文件夹中的所有图片文件
        image_files = [f for f in os.listdir(folder_path) if f.endswith('.jpg')]

        # 对文件名进行排序，以便按顺序处理图片
        image_files.sort()

        # 读取第一张图片
        first_image = cv2.imread(os.path.join(folder_path, image_files[0]))

        # 获取图片的形状
        image_shape = first_image.shape

        # 创建视频写入对象
        video_path = os.path.join(file2, 'pcd_overlook.mp4')
        video_writer = cv2.VideoWriter(video_path, cv2.VideoWriter_fourcc('m', 'p', '4', 'v'), hz,
                                       (image_shape[1], image_shape[0]))

        # 遍历图片文件列表
        for i in range(len(image_files) - 1):
            # 读取当前图片和下一张图片
            current_image = cv2.imread(os.path.join(folder_path, image_files[i]))
            next_image = cv2.imread(os.path.join(folder_path, image_files[i + 1]))

            # 获取当前图片和下一张图片的时间戳
            current_timestamp = float(image_files[i].split('.jpg')[0])
            next_timestamp = float(image_files[i + 1].split('.jpg')[0])

            # 计算时间间隔
            time_interval = next_timestamp - current_timestamp
            print(time_interval)

            # 根据实际时间间隔调整循环次数
            video_writer.write(current_image)
            if int(time_interval * hz) > 1.5:
                for _ in range(int(time_interval * hz)):
                    video_writer.write(current_image)

        # 释放资源
        video_writer.release()
        if os.path.exists(video_path):
            video_output_path = os.path.join(output_dir[0:-4], 'pcd_overlook.mp4')
            command = ["ffmpeg", "-i", video_path, "-c:v", "libx264", video_output_path]
            p = Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            p.communicate()
            os.remove(video_path)
            shutil.rmtree(file1)
            shutil.rmtree(file2)
            shutil.rmtree(file3)

    e_time = datetime.datetime.now()
    print("using " + str(e_time - s_time) + "s")
    # shutil.rmtree(file1)
    # shutil.rmtree(file2)
    # shutil.rmtree(output_dir)
    return output_dir[0:-4]
