# -*- coding: utf-8 -*-
import sys
reload(sys)
sys.setdefaultencoding('utf8')
import os
import time
import oss2
from rosbag import Bag, Compression
import logging
from utils import yaml_utils 
compress_way = Compression.BZ2
key2 = 'data/'
key3 = 'data_merge/'
# --------------------------------------------------
def merge(local_bags, merged_prefix, local_merged_dir, merged_bag_name):
    try:
        output_bag_file = local_merged_dir + merged_bag_name
        with Bag(output_bag_file, 'w', compression=compress_way) as o:
            for i in range(len(local_bags)):
                with Bag(local_bags[i], 'r') as ib:
                    for topic, msg, t in ib:
                        o.write(topic, msg, t)
        bucket.put_object_from_file(merged_prefix + merged_bag_name, output_bag_file)
        return output_bag_file
    except Exception as e:
        logging.exception("bag包合并报错: %s" % str(e))

if __name__ == '__main__':
    # --------------------------------------------------
    if len(sys.argv) != 2:
        print("Usage: python script.py <config_file_path>")
        sys.exit(1)
    config_file_path = sys.argv[1]
    try:
        config = yaml_utils.parse_yaml(config_file_path)
    except Exception as e:
        print("Error parsing YAML file:", e)
        sys.exit(1)
    # --------------------------------------------------
    temp_dir = config['temp-dir']
    log_file_path = config['log-file-path'] # 日志文件地址
    sleep_time = config['sleep-time']  # 每多少秒扫描一次
    auth = oss2.Auth(config['oss']['access-key-id'], config['oss']['access-key-secret']) # oss 账号密码
    endpoint = config['oss']['endpoint'] # oss 地址
    bucket_name = config['oss']['bucket-name'] # oss 桶
    oss_data_root = config['oss']['data-root'] # 数据所在的oss根目录
    # --------------------------------------------------
    logging.basicConfig(filename=log_file_path, level=logging.INFO,format='%(asctime)s - %(levelname)s - %(message)s')
    bucket = oss2.Bucket(auth, endpoint, bucket_name)
    # --------------------------------------------------
    logging.info("临时文件目录:%s", config['temp-dir'])
    logging.info("日志文件路径:%s", config['log-file-path'])
    logging.info("Sleep Time:%s", config['sleep-time'])
    logging.info("OSS 的 Access Key ID:%s", config['oss']['access-key-id'])
    logging.info("OSS 的 Access Key Secret:%s", config['oss']['access-key-secret'])
    logging.info("OSS 的 Endpoint:%s", config['oss']['endpoint'])
    logging.info("OSS 的桶:%s", config['oss']['bucket-name'])
    logging.info("OSS 的数据根目录:%s", config['oss']['data-root'])
    # --------------------------------------------------
    while True:
        logging.info("开始新一轮扫描")
        try:
            prefix_list = []
            # 2 获取已经上传完成的所有目录并分组
            upload_completed_prefix_list = []
            for obj1 in oss2.ObjectIterator(bucket, prefix=oss_data_root, delimiter='/'):
                bag_list = []
                target_path = ''
                if str(obj1.key).count('/') == 2:  # pujin/robot-01/
                    # 6 校验bag包个数
                    for obj2 in oss2.ObjectIterator(bucket, prefix=str(obj1.key) + key2, delimiter='/'):
                        if str(obj2.key).count(
                                '/') == 4:  # pujin/robot-01/data/2023-12-20-02-16-56_obstacledetection_10/
                            bag_need = str(obj2.key).split('_')[-1][:-1]
                            count_bag = 0
                            for obj3 in oss2.ObjectIterator(bucket, prefix=str(obj2.key)):
                                if obj3.key[-3:] == 'bag':
                                    count_bag = count_bag + 1
                            if bag_need == str(count_bag):
                                upload_completed_prefix_list.append(obj2.key)
            # 处理
            for prefix in upload_completed_prefix_list:
                local_delete_list = []
                oss_delete_list = []
                # 获取要处理的包
                oss_bags = []
                local_bags = []
                for obj in oss2.ObjectIterator(bucket, prefix=prefix):
                    if str(obj.key).endswith('.bag'):
                        oss_bags.append(str(obj.key))
                split_list = prefix.replace(key2, key3).split('/')
                merged_prefix = '/'.join(split_list[0:-2]) + '/'
                local_merged_dir = temp_dir + merged_prefix
                if not os.path.exists(local_merged_dir):
                    os.makedirs(local_merged_dir)
                merged_bag_name = split_list[-2]
                merged_bag_full_name = merged_bag_name + '.bag'
                local_bag_dir = temp_dir + '/'.join(split_list[0:-1]) + '/'
                if not os.path.exists(local_bag_dir):
                    os.makedirs(local_bag_dir)

                # 下载
                for oss_bag in oss_bags:
                    bag_name = str(oss_bag).split('/')[-1]
                    local_bag_path = local_bag_dir + bag_name
                    bucket.get_object_to_file(oss_bag, local_bag_path)
                    local_bags.append(local_bag_path)
                    local_delete_list.append(local_bag_path)
                    oss_delete_list.append(oss_bag)

                # 1 合并bag
                logging.info("合并文件，key前缀为: %s" % str(local_merged_dir))
                merged_bag_file_path = merge(local_bags, merged_prefix, local_merged_dir, merged_bag_full_name)
                local_delete_list.append(merged_bag_file_path)
                # 删除本地临时文件
                if len(local_delete_list) > 0:
                    for local_delete in local_delete_list:
                        try:
                            os.remove(local_delete)
                        except Exception as e:
                            logging.exception("删除本地临时文件报错: %s" % str(e))

                # 删除oss原始文件
                if len(oss_delete_list) > 0:
                    bucket.batch_delete_objects(oss_delete_list)
            time.sleep(sleep_time)
        except Exception as e:
            logging.exception("全局错误处理: %s" % str(e))
