# -*- coding: utf-8 -*-
# ------- 全局配置 -------
import os
import time 
import oss2
import xml.etree.ElementTree as ET
import shutil
import docker
import logging
from utils import json_utils
from utils import yaml_utils
# --------------------------------------------------
vehicle_name = 'PuJin_patrol_robot'  # 配送 PuJin_distribution 巡检 PuJin_patrol_robot
xoscName = 'scenario.xosc'
# --------------------------------------------------
def move_xosc_before_simulation(root_path):
    try:
        xosc_src = os.path.join(root_path, 'scenario_orig.xosc')
        xosc_target = os.path.join(root_path, 'openx0.xosc')
        tree1 = ET.parse(xosc_src)
        root1 = tree1.getroot()
        for node0 in root1:
            if node0.tag == 'RoadNetwork':
                for node1 in node0:
                    # 打印根元素的标签名
                    if 'LogicFile' == node1.tag:
                        xodr_src = str(node1.get('filepath'))
                        xodr_target = root_path + xodr_src.rsplit('/', 1)[-1]
                        shutil.copy(xodr_src, xodr_target)
                        node1.set('filepath', xodr_target)
                    if 'SceneGraphFile' == node1.tag:
                        osgb_src = str(node1.get('filepath'))
                        osgb_target = root_path + osgb_src.rsplit('/', 1)[-1]
                        shutil.copy(osgb_src, osgb_target)
                        node1.set('filepath', osgb_target)
            if node0.tag == 'Entities':
                for node1 in node0:
                    if node1.get("name") == 'Ego':
                        for node2 in node1:
                            if node2.tag == 'Vehicle':
                                node2.set('name', vehicle_name)
        tree1.write(xosc_target)
        # 检查文件是否存在，以避免在文件不存在时引发异常
        if os.path.exists(xosc_src):
            # 删除文件
            os.remove(xosc_src)
            print("文件已删除:", xosc_src)
        else:
            print("文件不存在:", xosc_src)

    except Exception as e:
        logging.exception("修改xosc报错: %s" % str(e))


def upload_simulation(parse_prefix, mp41):
    try:
        # 上传两次吧，对应上
        bucket.put_object_from_file(parse_prefix + 'scenario_orig.mp4', mp41)
        bucket.put_object_from_file(parse_prefix + 'scenario_hmi.mp4', mp41)
        logging.info('上传仿真视频到 %s' % parse_prefix + 'scenario_orig.mp4')
        shutil.rmtree(simulation_dir)  # 仿真完就删除
    except Exception as e:
        error_bag_list = json_utils.parse_json_to_string_array(error_bag_json)
        error_bag_list.append(parse_prefix)
        json_utils.list_to_json_file(error_bag_list, error_bag_json)
        logging.exception("上传视频报错 %s" % str(e))


def simulation(parse_prefix, mp41):
    try:
        os.system('docker start '+container_name)
        # 实例化Docker客户端
        client = docker.from_env()
        while True:
            time.sleep(5)
            # 获取容器列表
            containers = client.containers.list()
            run = False
            # 打印容器列表
            for container in containers:
                if container_name == container.name:
                    run = True
                    break
            if not run:
                break
        time.sleep(5)
        upload_simulation(parse_prefix, mp41)
    except Exception as e:
        logging.exception("生成仿真视频报错 %s" % str(e))


def is_upload_completed(bucket, prefix):
    target_number = str(prefix).split('_')[-1][:-1]
    count = 0
    for obj in oss2.ObjectIterator(bucket, prefix=prefix):
        if obj.key != prefix:
            count += 1
    return int(count) == int(target_number)


# ------- 获取合并之后的bag包，解析出csv -------
if __name__ == '__main__':
     # --------------------------解析配置文件------------------------
    if len(sys.argv) != 2:
        print("Usage: python script.py <config_file_path>")
        sys.exit(1)
    config_file_path = sys.argv[1]
    try:
        config = yaml_utils.parse_yaml(config_file_path)
    except Exception as e:
        print("Error parsing YAML file:", e)
        sys.exit(1)
    # ----------------------全局变量----------------------------
    oss_data_root = config['oss']['data-root'] # 数据所在的oss根目录
    simulation_dir = config['simulation-dir'] # 仿真目录
    endpoint = config['oss']['endpoint'] # oss 地址
    bucket_name = config['oss']['bucket-name'] # oss 桶
    temp_dir = config['temp-dir'] # 临时文件目录
    log_file_path = config['log-file-path'] # 日志文件地址
    error_bag_json = config['error-bag-json'] # 错误目录json
    sleep_time = config['sleep-time']  # 每多少秒扫描一次
    container_name = config['container-name']
    # --------------------------------------------------
    logging.basicConfig(filename=log_file_path, level=logging.INFO,format='%(asctime)s - %(levelname)s - %(message)s')
    auth = oss2.Auth(config['oss']['access-key-id'], config['oss']['access-key-secret']) # oss 账号密码
    bucket = oss2.Bucket(auth, endpoint, bucket_name)
    # --------------------------------------------------
    logging.info("OSS 数据根目录:%s", config['oss']['data-root'])
    logging.info("仿真目录:%s", config['simulation-dir'])
    logging.info("临时文件目录:%s", config['temp-dir'])
    logging.info("Access Key ID:%s", config['oss']['access-key-id'])
    logging.info("Access Key Secret:%s", config['oss']['access-key-secret'])
    logging.info("Endpoint:%s", config['oss']['endpoint'])
    logging.info("Bucket Name:%s", config['oss']['bucket-name'])
    logging.info("日志文件目录:%s", config['log-file-path'])
    logging.info("错误目录json路径:%s", config['error-bag-json'])
    logging.info("每一轮休眠时间:%s", config['sleep-time'])
    logging.info("VTD容器名称:%s", config['container-name'])
    # --------------------------------------------------
    while True:
        try:
            logging.info("开始新一轮扫描")
            oss_delete_list = []
            prefix_list = []
            # 2 获取已经上传完成的所有目录并分组
            for obj1 in oss2.ObjectIterator(bucket, prefix=oss_data_root):
                try:
                    # 获取合并后的包
                    merged_bag_object_key = str(obj1.key)
                    if 'data_merge' in str(obj1.key) and str(obj1.key).endswith('.bag'):
                        merged_bag_object_key_split = merged_bag_object_key.split('/')
                        merged_prefix = '/'.join(merged_bag_object_key_split[:-1])
                        parse_prefix = merged_prefix.replace('data_merge', 'data_parse')
                        parse_prefix_full = merged_bag_object_key.replace('data_merge', 'data_parse')[:-4] + '/'
                        xosc_done = False
                        mp4_done = False
                        for obj3 in oss2.ObjectIterator(bucket, prefix=str(parse_prefix_full)):
                            if xoscName in str(obj3.key):
                                xosc_done = True
                            if '/scenario_orig.mp4' in str(obj3.key):
                                mp4_done = True
                        if not xosc_done:
                            continue
                        if mp4_done:
                            continue
                        error_bag_list = json_utils.parse_json_to_string_array(error_bag_json)
                        if str(parse_prefix_full) in error_bag_list:
                            continue
                        logging.info("需要生成仿真视频: %s" % str(parse_prefix_full))
                        parse_prefix_full = str(parse_prefix_full)
                        local_dir_full = temp_dir + parse_prefix_full
                        if not os.path.exists(local_dir_full):
                            os.makedirs(local_dir_full)
                        # 下载两个csv
                        root_path1 = local_dir_full + 'orig/'
                        if not os.path.exists(root_path1):
                            os.makedirs(root_path1)
                        bucket.get_object_to_file(parse_prefix_full + xoscName, root_path1 + 'scenario_orig.xosc')
                        move_xosc_before_simulation(root_path1)
                        simulation(parse_prefix_full, root_path1 + 'simulation.mp4')
                except Exception as e:
                    logging.exception("局部异常处理: %s", str(e))
            time.sleep(sleep_time)
        except Exception as e:
            logging.exception("全局异常处理: %s", str(e))
